@@ -3,15 +3,14 @@ module Asciidoctor
 # A module for defining converters that are used to convert {AbstractNode} objects in a parsed AsciiDoc document to an
 # output (aka backend) format such as HTML or DocBook.
 #
-# Implementing a custom converter involves:
+# A {Converter} is typically instantiated each time an AsciiDoc document is processed (i.e., parsed and converted).
+# Implementing a custom converter entails:
 #
 # * Including the {Converter} module in a converter class and implementing the {Converter#convert} method or extending
 #   the {Converter::Base Base} class and implementing the dispatch methods that map to each node.
 # * Optionally registering the converter with one or more backend names statically using the +register_for+ DSL method
 #   contributed by the {Converter::Config Config} module.
 #
-# {Converter} instances are typically instantiated for each instance of a parsed AsciiDoc document.
-#
 # Examples
 #
 #   class TextConverter
@@ -32,7 +31,7 @@ module Asciidoctor
 #       end
 #     end
 #   end
-#   puts Asciidoctor.convert_file 'sample.adoc', backend: :text
+#   puts Asciidoctor.convert_file 'sample.adoc', backend: :text, safe: :safe
 #
 #   class Html5Converter < (Asciidoctor::Converter.for 'html5')
 #     register_for 'html5'
@@ -40,7 +39,7 @@ module Asciidoctor
 #       %(<p>#{node.content}</p>)
 #     end
 #   end
-#   puts Asciidoctor.convert_file 'sample.adoc'
+#   puts Asciidoctor.convert_file 'sample.adoc', safe: :safe
 module Converter
   autoload :CompositeConverter, %(#{__dir__}/converter/composite)
   autoload :TemplateConverter, %(#{__dir__}/converter/template)
@@ -83,6 +82,23 @@ module Converter
     true
   end
 
+  # Public: Derive backend traits (basebackend, filetype, outfilesuffix, htmlsyntax) from the given backend.
+  #
+  # backend - the String backend from which to derive the traits
+  #
+  # Returns the backend traits for the given backend as a [Hash].
+  def self.derive_backend_traits backend
+    return {} unless backend
+    if (t_outfilesuffix = DEFAULT_EXTENSIONS[(t_basebackend = backend.sub TrailingDigitsRx, '')])
+      t_filetype = t_outfilesuffix.slice 1, t_outfilesuffix.length
+    else
+      t_outfilesuffix = %(.#{t_filetype = t_basebackend})
+    end
+    t_filetype == 'html' ?
+      { basebackend: t_basebackend, filetype: t_filetype, htmlsyntax: 'html', outfilesuffix: t_outfilesuffix } :
+      { basebackend: t_basebackend, filetype: t_filetype, outfilesuffix: t_outfilesuffix }
+  end
+
   module BackendTraits
     def basebackend value = nil
       value ? (backend_traits[:basebackend] = value) : backend_traits[:basebackend]
@@ -113,25 +129,14 @@ module Converter
     end
 
     def backend_traits
-      @backend_traits ||= derive_backend_traits
+      @backend_traits ||= Converter.derive_backend_traits @backend
     end
 
     alias backend_info backend_traits
 
-    private def derive_backend_traits
-      BackendTraits.derive_backend_traits @backend
-    end
-
+    # Deprecated: Use {Converter.derive_backend_traits} instead.
     def self.derive_backend_traits backend
-      return {} unless backend
-      if (t_outfilesuffix = DEFAULT_EXTENSIONS[(t_basebackend = backend.sub TrailingDigitsRx, '')])
-        t_filetype = t_outfilesuffix.slice 1, t_outfilesuffix.length
-      else
-        t_outfilesuffix = %(.#{t_filetype = t_basebackend})
-      end
-      t_filetype == 'html' ?
-        { basebackend: t_basebackend, filetype: t_filetype, htmlsyntax: 'html', outfilesuffix: t_outfilesuffix } :
-        { basebackend: t_basebackend, filetype: t_filetype, outfilesuffix: t_outfilesuffix }
+      Converter.derive_backend_traits backend
     end
   end
 
@@ -142,7 +147,7 @@ module Converter
     # backends - One or more String backend names with which to associate this {Converter} class.
     #
     # Returns nothing.
-    private def register_for *backends
+    def register_for *backends
       Converter.register self, *backends
     end
   end
@@ -241,7 +246,9 @@ module Converter
       registry.merge
     end
 
-    private def registry
+    private
+
+    def registry
       raise ::NotImplementedError, %(#{Factory} subclass #{self.class} must implement the ##{__method__} method)
     end
   end
@@ -252,10 +259,10 @@ module Converter
     def initialize seed_registry = nil
       if seed_registry
         seed_registry.default = seed_registry.delete '*'
+        @registry = seed_registry
       else
-        seed_registry = {}
+        @registry = {}
       end
-      @registry = seed_registry
     end
 
     # Public: Unregister all Converter classes that are registered with this factory. Intended for testing only.
@@ -306,11 +313,11 @@ module Converter
 
       def for backend
         @@registry.fetch backend do
-          PROVIDED[backend] ? @@mutex.synchronize do
+          PROVIDED[backend] ? (@@mutex.synchronize do
             # require is thread-safe, so no reason to refetch
             require PROVIDED[backend]
             @@registry[backend]
-          end : catch_all
+          end) : catch_all
         end
       end
 
@@ -344,7 +351,9 @@ module Converter
         @registry.fetch(backend) { super }
       end
 
-      private def catch_all
+      private
+
+      def catch_all
         @registry.default || super
       end
     end
@@ -357,9 +366,9 @@ module Converter
   #
   # Returns nothing.
   private_class_method def self.included into
-    into.include BackendTraits
+    into.send :include, BackendTraits
     into.extend Config
-  end
+  end || :included
 
   # An abstract base class for defining converters that can be used to convert {AbstractNode} objects in a parsed
   # AsciiDoc document to a backend format such as HTML or DocBook.
