@@ -9,7 +9,7 @@ module Helpers
   # passes a message to Kernel#raise if on_failure is :abort or Kernel#warn if
   # on_failure is :warn to communicate to the user that processing is being
   # aborted or functionality is disabled, respectively. If a gem_name is
-  # specified, the message communicates that a required gem is not installed.
+  # specified, the message communicates that a required gem is not available.
   #
   # name       - the String name of the library to require.
   # gem_name   - a Boolean that indicates whether this library is provided by a RubyGem,
@@ -159,7 +159,7 @@ module Helpers
 
   # Public: Removes the file extension from filename and returns the result
   #
-  # filename - The String file name to process
+  # filename - The String file name to process; expected to be a posix path
   #
   # Examples
   #
@@ -168,7 +168,11 @@ module Helpers
   #
   # Returns the String filename with the file extension removed
   def self.rootname filename
-    filename.slice 0, ((filename.rindex '.') || filename.length)
+    if (last_dot_idx = filename.rindex '.')
+      (filename.index '/', last_dot_idx) ? filename : (filename.slice 0, last_dot_idx)
+    else
+      filename
+    end
   end
 
   # Public: Retrieves the basename of the filename, optionally removing the extension, if present
@@ -188,12 +192,48 @@ module Helpers
   # Returns the String filename with leading directories removed and, if specified, the extension removed
   def self.basename filename, drop_ext = nil
     if drop_ext
-      ::File.basename filename, (drop_ext == true ? (::File.extname filename) : drop_ext)
+      ::File.basename filename, (drop_ext == true ? (extname filename) : drop_ext)
     else
       ::File.basename filename
     end
   end
 
+  # Public: Returns whether this path has a file extension.
+  #
+  # path - The path String to check; expects a posix path
+  #
+  # Returns true if the path has a file extension, false otherwise
+  def self.extname? path
+    (last_dot_idx = path.rindex '.') && !(path.index '/', last_dot_idx)
+  end
+
+  # Public: Retrieves the file extension of the specified path. The file extension is the portion of the path in the
+  # last path segment starting from the last period.
+  #
+  # This method differs from File.extname in that it gives us control over the fallback value and is more efficient.
+  #
+  # path     - The path String in which to look for a file extension
+  # fallback - The fallback String to return if no file extension is present (optional, default: '')
+  #
+  # Returns the String file extension (with the leading dot included) or the fallback value if the path has no file extension.
+  if ::File::ALT_SEPARATOR
+    def self.extname path, fallback = ''
+      if (last_dot_idx = path.rindex '.')
+        (path.index '/', last_dot_idx) || (path.index ::File::ALT_SEPARATOR, last_dot_idx) ? fallback : (path.slice last_dot_idx, path.length)
+      else
+        fallback
+      end
+    end
+  else
+    def self.extname path, fallback = ''
+      if (last_dot_idx = path.rindex '.')
+        (path.index '/', last_dot_idx) ? fallback : (path.slice last_dot_idx, path.length)
+      else
+        fallback
+      end
+    end
+  end
+
   # Internal: Make a directory, ensuring all parent directories exist.
   def self.mkdir_p dir
     unless ::File.directory? dir
